@@ -68,6 +68,9 @@ class SearchQuery(str):
 
         if not definition.operation == OperationType.QUERY:
             raise ValueError("Needs to be operation")
+        
+
+        assert len(definition.variable_definitions) >= 2, "At least two arguments should be provided ($search: String, $values: [ID]))"
 
         if (
             len(definition.variable_definitions) > 0
@@ -76,6 +79,14 @@ class SearchQuery(str):
             raise ValueError(
                 "First parameter of search function should be '$search: String' if you provide arguments for your options. This parameter will be filled with userinput"
             )
+        
+        if (
+            len(definition.variable_definitions) > 0
+            and definition.variable_definitions[1].variable.name.value != "values"
+        ):
+            raise ValueError(
+                "Seconrd parameter of search function should be '$values: [ID]' if you provide arguments for your options. This parameter will be filled with the default values"
+            )
 
         wrapped_query = definition.selection_set.selections[0]
 
